@@ -485,6 +485,7 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 	const struct gsm_mncc_number *called;
 	struct call *call;
 	struct mncc_call_leg *leg;
+	struct osmo_gcr_parsed gcr;
 
 	if (rc < sizeof(*data)) {
 		LOGP(DMNCC, LOGL_ERROR, "gsm_mncc of wrong size %d vs. %zu\n",
@@ -525,6 +526,16 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 		return;
 	}
 
+	/* Decode the Global Call Reference (if present) */
+	if (data->fields & MNCC_F_GCR) {
+		if (osmo_dec_gcr(&gcr, data->gcr, sizeof(data->gcr)) < 0) {
+			LOGP(DMNCC, LOGL_ERROR,
+				"MNCC leg(%u) failed to parse GCR\n", data->callref);
+			mncc_send(conn, MNCC_REJ_REQ, data->callref);
+			return;
+		}
+	}
+
 	/* Create an RTP port and then allocate a call */
 	call = call_mncc_create();
 	if (!call) {
@@ -547,6 +558,11 @@ static void check_setup(struct mncc_connection *conn, const char *buf, int rc)
 	memcpy(&leg->calling, &data->calling, sizeof(leg->calling));
 	memcpy(&leg->imsi, data->imsi, sizeof(leg->imsi));
 
+	if (data->fields & MNCC_F_GCR) {
+		leg->base.call->gcr_present = true;
+		leg->base.call->gcr = gcr;
+	}
+
 	LOGP(DMNCC, LOGL_INFO,
 		"Created call(%u) with MNCC leg(%u) IMSI(%.16s)\n",
 		call->id, leg->callref, data->imsi);
@@ -871,6 +887,7 @@ int mncc_create_remote_leg(struct mncc_connection *conn, struct call *call)
 {
 	struct mncc_call_leg *leg;
 	struct gsm_mncc mncc = { 0, };
+	struct msgb *msg;
 	int rc;
 
 	leg = talloc_zero(call, struct mncc_call_leg);
@@ -916,6 +933,16 @@ int mncc_create_remote_leg(struct mncc_connection *conn, struct call *call)
 		OSMO_STRLCPY_ARRAY(mncc.called.number, call->dest);
 	}
 
+	/* Encode the Global Call Reference (if present) */
+	if (call->gcr_present) {
+		msg = msgb_alloc(sizeof(mncc.gcr), "MNCC GCR");
+		if (msg == NULL || (rc = osmo_enc_gcr(msg, &call->gcr)) == 0)
+			LOGP(DMNCC, LOGL_ERROR, "MNCC leg(%u) failed to encode GCR\n", call->id);
+		else
+			memcpy(&mncc.gcr[0], msg->data, rc);
+		msgb_free(msg);
+	}
+
 	/*
 	 * TODO/FIXME:
 	 *  - Determine/request channel based on offered audio codecs
